#!/bin/sh

###################################################
# Copyright 2015 -2018 VMware, Inc.  All rights reserved.
###################################################


if [ "`id -u`" != "0" ]; then
   #
   # Requires permission to access the XAUTHORITY file.
   #
   echo "StartSession.sh must be executed as superuser"
   exit 1
fi

#
# Default display resolution and depth.
#
xres=1280
yres=1024
bpp=24
keyboardLayout=
language=
enableFips=FALSE
enableHelpDesk=FALSE

#
# Check display manager
#
checkDM() {
   dm=
   if [ -z "$dm" ]; then
      ps -e -o comm= | grep "^gdm-binary$"
      [ "$?" = "0" ] && dm=gdm
   fi
   if [ -z "$dm" ]; then
      ps -e -o comm= | grep "^gdm$"
      [ "$?" = "0" ] && dm=gdm
   fi
   if [ -z "$dm" ]; then
      ps -e -o comm= | grep "^lightdm$"
      [ "$?" = "0" ] && dm=lightdm
   fi
}


initialise() {
   res="${xres}x${yres}"

   ps -e -o comm= | grep "^Xvfb$"
   if [ "$?" = "0" ]; then
      configure_xvfb
   else
      configure_xorg ${res}
   fi
}


configure_xvfb() {
   if [ "$dm" != "lightdm" ]; then
      exit 1
   fi

   XVFBWRAP=/usr/lib/vmware/viewagent/bin/Xvfb-lightdm-wrapper.sh

   # VMwareBlastServer + Xvfb only appear to work at 24bpp colour depth.
   #res="${res}x${bpp}"

   #
   # Do not stop and restart display manager if resolution is unchanged
   #
   # VMwareBlastServer + Xvfb only appear to work at 24bpp colour depth.
   #grep " ${res}\$" $XVFBWRAP >/dev/null 2>&1
   grep " ${res}x" $XVFBWRAP >/dev/null 2>&1
   if [ "$?" = "1" ]; then
      #
      # Requested Horizon Client resolution differs from current configuration,
      # for Xvfb this is nothing else for it but stop the X server, reconfigure
      # and restart.
      #
      initctl stop lightdm
      # VMwareBlastServer + Xvfb only appear to work at 24bpp colour depth.
      #sed -i -e "/-screen/s/-screen 0.*/-screen 0 $res/" $XVFBWRAP
      sed -i -e "/-screen/s/-screen 0.*/-screen 0 ${res}x24/" $XVFBWRAP
      initctl start lightdm
   fi
}


#
# Main
#

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
define_constants
identify_distribution

#
# Enable core dumps
#
ulimit -c unlimited

while getopts x:y:d:e:m:u:c:p:k:l:f:st o
do
   case $o in
   x)
      xres="$OPTARG";;
   y)
      yres="$OPTARG";;
   d)
      bpp="$OPTARG";;
   e)
      env="$OPTARG";;
   m)
      port="-m $OPTARG";;
   c)
      sslCiphers="--sslCiphers $OPTARG";;
   p)
      sslProtocols="--sslProtocols $OPTARG";;
   s)
      sslOptions="--sslCipherServerPreference";;
   t)
      enableHelpDesk="TRUE";;
   k)
      keyboardLayout="$OPTARG";;
   l)
      language="$OPTARG";;
   f)
      enableFips="TRUE";;
   esac
done



#
# Already running?
#
pidof VMwareBlastServer >/dev/null 2>&1
if [ "$?" = "0" ]; then
   exit 17
fi

if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ]; then
   pidof gnome-session >/dev/null 2>&1 || pidof gnome-session-binary >/dev/null 2>&1
   if [ "$?" != "0" ]; then
      echo "The gnome/mate desktop session doesn't exist"
      exit 18
   fi
fi

if ! getent passwd ${BLAST_ACCOUNT} >/dev/null; then
   echo "The blast server account ${BLAST_ACCOUNT} doesn't exist"
   exit 18
fi

if [ -n "$env" ]; then
   export $env
else
   extract_X_args
fi

initialise

directory_validity_check $BLAST_RUNTIME_DIR $BLAST_ACCOUNT $BLAST_GROUP 700

isRedhatOrCentOS6="FALSE"
majorVersion=`identify_distribution -r | head -c 1`
if [ $majorVersion = "6" ]; then
   distroID=$(identify_distribution -i)
   if [ "$distroID" = "CentOS" ] || [ "$distroID" = "RedHatEnterpriseClient" ] || [ "$distroID" = "RedHatEnterpriseServer" ] || [ "$distroID" = "RedHatEnterpriseWorkstation" ]; then
      isRedhatOrCentOS6="TRUE"
   fi
fi

# Part of gnome-screensaver bug on Ubuntu
isUbuntu="FALSE"
distroID=$(identify_distribution -i)
if [ "$distroID" = "Ubuntu" ]; then
   isUbuntu="TRUE"
fi

# SLED11 also has screen-saver autofit issue (Bug 1923479)
isSLED11="FALSE"
majorVersion=$(identify_distribution -r)
if [ $majorVersion = "11" ]; then
   distroID=$(identify_distribution -i)
   if [ "$distroID" = "SUSE LINUX" ]; then
      isSLED11="TRUE"
   fi
fi

xauthfile=$BLAST_RUNTIME_DIR/.xauth$DISPLAY
[ -f "$xauthfile" ] && rm -f $xauthfile
#
# Workaround for a system issue on SLED
# On SLED, hostname may changes by NetworkManager during Xorg is running
# In such case, there maybe no entry which matches current hostname in the database
# pointed by XAUTHORITY environment
# We'll check if there is such case, if yes, a new entry matches current host name will be
# added into the database
#
if [ "$DISTRO_ID" = "$DISTRO_ID_SUSE" ]; then
   XAUTH_LIST=$(xauth list $(hostname)/unix${DISPLAY})
   if [ -z "$XAUTH_LIST" ]; then
      XAUTH_COOKIE=$(xauth list localhost/unix${DISPLAY} | cut -f 5 -d " " | head -n 1)
      if [ -n "$XAUTH_COOKIE" ]; then
         xauth add "$(hostname)/unix${DISPLAY}" MIT-MAGIC-COOKIE-1 "$XAUTH_COOKIE" >/dev/null 2>&1
      fi
   fi
fi

#
# If the X Server is not running
# We need to notice user to check the graphic settings
#

if [ "$DISPLAY" = "" ]; then
   echo "Failed to start VMWareBlastServer since X Server process is not running. Please check your graphic settings."
   exit 20
fi

xauth extract $xauthfile $DISPLAY
if [ ! -f "$xauthfile" ]; then
   #
   # Workaround for a system issue.
   # On the CentOS and Redhat 6.x, it is possible that xauth failed for the SELinux security context.
   # Here we need to change the SELinux security context to workaround for the system issue.
   #
   if [ "$isRedhatOrCentOS6" = "TRUE" ]; then
      command -v getenforce >/dev/null 2>&1 && [ `getenforce` != "Disabled" ] && \
      chcon -u system_u -r object_r -t admin_home_t -l s0 $BLAST_RUNTIME_DIR
      xauth extract $xauthfile $DISPLAY

      # Restore the security context.
      command -v getenforce >/dev/null 2>&1 && [ `getenforce` != "Disabled" ] && \
      chcon -u system_u -r object_r -t initrc_tmp_t -l s0 $BLAST_RUNTIME_DIR

      if [ ! -f "$xauthfile" ]; then
         echo "Failed to create the xauthority file even change the SELinux security context, exit"
         exit 18
      fi
   else
      echo "Failed to create the xauthority file, exit"
      exit 18
   fi
fi
chown $BLAST_ACCOUNT $xauthfile
chmod 700 $xauthfile
export XAUTHORITY=$xauthfile

#
# We need to check the XServer is running or not.
# If the XServer is not running, the BlastServer will panic as the XOpenDisplay will fail.
# The panic will affect the user experience.
# To prevent from the panic, we need to check the XServer is running or not befor launch the BlastServer.
#
xset q >/dev/null 2>&1
if [ "$?" != "0" ]; then
   echo "XServer is not running, exit"
   exit 19
fi

#
# Read Token from stdin
#
Token=`cat -`
randomHandShakeFlag="-r "

#
# Workaround for the screen saver bug for Redhat 6.x, CentOS 6.x, Ubuntu and SLED11.
# Disable the resizing when the screen is locked.
#
if [ $isRedhatOrCentOS6 = "TRUE" ] || [ $isUbuntu = "TRUE" ] || [ $isSLED11 = "TRUE" ]; then
   disableResizeWhenScreenLocked="TRUE"
else
   disableResizeWhenScreenLocked="FALSE"
fi

export LD_LIBRARY_PATH="$LD_LIBRARY_PATH:/usr/lib/vmware/viewagent/VMwareBlastServer"
export HOME=/tmp/vmware-root

echo $Token | /usr/lib/vmware/viewagent/VMwareBlastServer/VMwareBlastServer ${port}  ${randomHandShakeFlag} \
   ${sslProtocols} ${sslCiphers} ${sslOptions} \
   -v -b ${BLAST_PORT} -u ${BLAST_ACCOUNT} -c \
   -s "RemoteDisplay.allowMultimon=TRUE" \
   -s "RemoteDisplay.allowSoftwareH264=TRUE" \
   -s "RemoteDisplay.allowVMWServerHeartbeat=TRUE" \
   -s "RemoteDisplay.allowSessionCloseMessage=TRUE" \
   -s "RemoteDisplay.allowRequestStreamState=TRUE" \
   -s "RemoteDisplay.heartbeatInterval=120" \
   -s "mksVNCServer.disableResizeWhenScreenLocked=$disableResizeWhenScreenLocked" \
   -s "mksVNCServer.lastRootSizeCheck=TRUE" \
   -s "mksVNCServer.initialClippedWidth=2560" \
   -s "mksVNCServer.initialClippedHeight=1600" \
   -s "mksVNCServer.checkPulseAudio=TRUE" \
   -s "mks.panicOnTopologyFail=TRUE" \
   -s "mks.enableMKSControl=TRUE" \
   -s "mks.enableFIPSMode=${enableFips}" \
   -s "mks.enableHelpDesk=${enableHelpDesk}" \
   -s "mks.x.skipCoreDumpOnXIOError=TRUE" &
ret=$?
blastpid=$!

#
# Record the PID of VMwareBlastServer to temporary file
# It will be used/removed by CheckBlastServerStatus.sh
#
VMWARE_ROOT_PATH=/tmp/vmware-root
directory_validity_check $VMWARE_ROOT_PATH root root 0
echo $blastpid > /tmp/vmware-root/pidofvmwareblast

#
# Monitor the pulseaudio to prepare the socket and cookie.
#
/usr/lib/vmware/viewagent/bin/MonitorAudio.sh $blastpid &

#
# set client keyboard layout into agent
#
LOCALE_PROFILE_PATH=/etc/profile.d/vmware-lang.sh
if [ -f "$LOCALE_PROFILE_PATH" ];  then
   # remove the lang first if it exists.
   rm -f "$LOCALE_PROFILE_PATH"
fi

if [ ! -z "${keyboardLayout}" ]; then
   KEYBOARD_LAYOUT=/var/log/vmware/keyboardLayout
   originalLayout=`setxkbmap -print -verbose 10 | grep layout | awk  '/layout/ {print $2}'`
   touch $KEYBOARD_LAYOUT
   echo "$originalLayout $keyboardLayout" > $KEYBOARD_LAYOUT
   setxkbmap $keyboardLayout
fi

if [ ! -z "${language}" ]; then
   # check if input arg language is supported by this agent.
   lang=`locale -a | grep -i "${language}"`
   if [ ! -z "${lang}" ]; then
      touch "$LOCALE_PROFILE_PATH"
      echo export LANG="${language}" >> "$LOCALE_PROFILE_PATH"
   fi
fi

exit $ret
