#!/bin/bash

###################################################
# Copyright 2017 - 2018 VMware, Inc.  All rights reserved.
###################################################


ngvcAgentPrefix="`date` - [NGVC-Agent-Script]"
logPath="/var/log/vmware/viewagent-ngvc.log"
customizationState="success"
customizationScriptFolder="/var/userScript"

checkMachinePassword() {
   local checksum
   local passwdsum

   if [ -z "$MachinePassword" ]; then
      echo "${ngvcAgentPrefix}: checkMachinePassword: Machine password not specified." >> $logPath
      return 1
   fi

   checksum=`vmware-rpctool "info-get guestinfo.machinePasswdChecksum"`
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: checkMachinePassword: Failed to fetch checksum."  >> $logPath
      return 0
   fi

   if [ "$checksum" = "none" ]; then
      echo "${ngvcAgentPrefix}: checkMachinePassword: checksum is none, ignore machine password checksum."  >> $logPath
      return 0
   fi

   passwdsum=`echo -n $MachinePassword | iconv -f UTF-8 -t UTF-16LE | openssl dgst -sha256 -binary | base64`
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: checkMachinePassword: Failed to calulate summary."  >> $logPath
      return 1
   fi

   if [ "$checksum" != "$passwdsum" ]; then
      echo "${ngvcAgentPrefix}: checkMachinePassword: The checksum doesn't equal to password summary."  >> $logPath
      return 1
   fi

   return 0
}

acquireMachinePassword() {
   local obfuscated
   local hostname
   local uuid
   local digest
   local keyvalue
   local ivvalue

   obfuscated=`vmware-rpctool "info-get guestinfo.machinePasswd"`
   if [ "$?" != "0" ] || [ "$obfuscated" = "none" ]; then
      echo "${ngvcAgentPrefix}: acquireMachinePassword: Failed to fetch obfuscated password."  >> $logPath
      return 1
   fi

   hostname=`vmware-rpctool "info-get guestinfo.hostName"`
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: acquireMachinePassword: Failed to fetch hostname."  >> $logPath
      return 1
   fi

   uuid=`vmware-rpctool "info-get guestinfo.cloneprep.clone.uuid"`
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: acquireMachinePassword: Failed to fetch cloneprep uuid."  >> $logPath
      return 1
   fi

   digest=`echo -n "$hostname$uuid" | openssl dgst -sha256 -r`
   if [ "$?" != "0" ] || [ ${#digest} -lt 64 ]; then
      digest=`echo -n "$hostname$uuid" | openssl dgst -sha256`
      if [ "$?" != "0" ] || [ ${#digest} -lt 64 ]; then
         echo "${ngvcAgentPrefix}: acquireMachinePassword: Failed to calulate key digest."  >> $logPath
         return 1
      fi
   fi

   keyvalue=${digest:0:32}
   ivvalue=${digest:32:32}

   MachinePassword=`echo -n $obfuscated |base64 -d | openssl enc -d -aes-128-cfb -K $keyvalue -iv $ivvalue`
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: acquireMachinePassword: Failed to unobfuscate machine password."  >> $logPath
      return 1
   fi

   checkMachinePassword
   return $?
}

offlineDomainJoin() {
   local fqdn
   local hostName
   local upFqdn
   local lowHostFqdn
   local samAccountName

   REG_TOOL=/opt/pbis/bin/regshell
   if [ -f $REG_TOOL ]; then
      MachinePassword=
      acquireMachinePassword
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to acquire machine password."  >> $logPath
         return 1
      fi

      hostName=`vmware-rpctool "info-get guestinfo.hostName"`
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to acquire hostname."  >> $logPath
         return 1
      fi

      fqdn=`vmware-rpctool "info-get guestinfo.primed.fqdn"`
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to acquire fqdn."  >> $logPath
         return 1
      fi

      typeset -u samAccountName
      if [ "$?" = "0" ]; then
         samAccountName="$hostName\$"
      else
         samAccountName=$(echo -n "$hostName\$" | tr '[a-z]' '[A-Z]')
      fi
      typeset -u upFqdn
      if [ "$?" = "0" ]; then
         upFqdn="$fqdn"
      else
         upFqdn=$(echo -n "$fqdn" | tr '[a-z]' '[A-Z]')
      fi
      typeset -l lowHostFqdn
      if [ "$?" = "0" ]; then
         lowHostFqdn="$hostName.$fqdn"
      else
         lowHostFqdn=$(echo -n "$hostName.$fqdn" | tr '[A-Z]' '[a-z]')
      fi

      $REG_TOOL list_keys [HKEY_THIS_MACHINE\\Services\\lsass\\Parameters\\Providers\\ActiveDirectory\\DomainJoin\\$upFqdn\\Pstore]
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to list keys in PBIS. Please join master VM in domain."  >> $logPath
         return 1
      fi

      $REG_TOOL set_value [HKEY_THIS_MACHINE\\Services\\lsass\\Parameters\\Providers\\ActiveDirectory\\DomainJoin\\$upFqdn\\Pstore] "Fqdn" "$lowHostFqdn"
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to set Fqdn in PBIS."  >> $logPath
         return 1
      fi

      $REG_TOOL set_value [HKEY_THIS_MACHINE\\Services\\lsass\\Parameters\\Providers\\ActiveDirectory\\DomainJoin\\$upFqdn\\Pstore] "SamAccountName" "$samAccountName"
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to set SamAccountName in PBIS."  >> $logPath
         return 1
      fi

      $REG_TOOL set_value [HKEY_THIS_MACHINE\\Services\\lsass\\Parameters\\Providers\\ActiveDirectory\\DomainJoin\\$upFqdn\\Pstore\\PasswordInfo] "Password" "$MachinePassword"
      if [ "$?" != "0" ]; then
         echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to set machine password in PBIS."  >> $logPath
         return 1
      fi

      for count in $( seq 1 3 )
      do
         ifconfig -a |grep inet |grep -v 127.0.0.1 |grep -v inet6 > /dev/null 2>&1
         if [ "$?" = "0" ]; then
            service lwsmd restart
            pbis status > /dev/null 2>&1
            if [ "$?" = "0" ]; then
               break
            fi
         fi
         sleep 3
      done

      sleep 3

      # Init the machine account
      KINIT_TOOL=/opt/pbis/bin/kinit
      if [ "$?" = "0" ] && [ -f $KINIT_TOOL ]; then
         echo -n "$MachinePassword" | $KINIT_TOOL $samAccountName
         if [ "$?" != "0" ]; then
            echo "${ngvcAgentPrefix}: offlineDomainJoin: Failed to init machine account with kinit." >> $logPath
         fi
      fi
   else
      echo "${ngvcAgentPrefix}: offlineDomainJoin: PBIS doesn't install, please install PBIS to join domain in master VM."  >> $logPath
      return 1
   fi

   return 0
}

changeHostName() {
   echo "${ngvcAgentPrefix}: Beginning changing host name..." >> $logPath
   oldHostname=`hostname`
   hostName=`vmware-rpctool "info-get guestinfo.hostName"`

   case "$DISTRO_ID" in
   ${DISTRO_ID_UBUNTU})
      hostname $hostName
      bash -c "echo $hostName > /etc/hostname"
      sed -i "s/$oldHostname/$hostName/g" /etc/hosts
      ;;
   ${DISTRO_ID_SUSE})
      hostname $hostName
      bash -c "echo $hostName > /etc/HOSTNAME"
      sed -i "s/$oldHostname/$hostName/g" /etc/hosts
      ;;
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      if [ ${OS_MAJOR} -ge "7" ]; then
         hostname $hostName
         bash -c "echo $hostName > /etc/hostname"
         sed -i "s/$oldHostname/$hostName/g" /etc/hosts
      else
         # redhat 6
         sed -i "s/$oldHostname/$hostName/g" /etc/sysconfig/network
         sed -i "s/$oldHostname/$hostName/g" /etc/hosts
      fi
      ;;
   esac
}

changeHWAddress() {
   echo "${ngvcAgentPrefix}: Begin changing MAC..." >> $logPath
   # change the mac addrss for each network adapter
   netCount=`ifconfig | grep HWaddr | wc -l`
   netIdx=0
   while [ $netIdx -lt $netCount ]
   do
      macAddrStr="info-get guestinfo.fork.ethernet${netIdx}.address"
      mac=`vmware-rpctool "$macAddrStr"`

      netIdx=`expr $netIdx + 1`
      netif=`ifconfig | grep HWaddr | awk '{print $1}' | sed -n ${netIdx}p`
      ifconfig $netif down
      ifconfig $netif hw ether $mac
      ifconfig $netif up

   done
   echo "${ngvcAgentPrefix}: Complete changing MAC..." >> $logPath
}

runPostCustomizationScript() {
   # run post-customize script
   postCustomize=`vmware-rpctool "info-get guestinfo.scripts.PostCustomization"`
   if [ -n "$postCustomize" ]; then
      echo "${ngvcAgentPrefix}: PostCustomization: $postCustomize.." >> $logPath
      scriptPath=`echo $postCustomize | awk '{print $1}' | sed 's/\"//g'`

      # script folder should be under /etc/vmware which has root:root privilege
      scriptfolder=$(dirname "${scriptPath}")
      if [ "$customizationScriptFolder" = "$scriptfolder" ]; then
         if [ -f "$scriptPath" ]; then
             scriptWithPara=`echo $postCustomize | sed 's/\"//g'`
             owner=`stat -c "%U" "$scriptPath"`
             permission=`stat -c "%a" "$scriptPath"`
             if [ "$owner" = "root" ] && [ "$permission" = "700" ]; then
                echo "${ngvcAgentPrefix}: Begining Post Customization script command: ${scriptWithPara}" >> $logPath
                $scriptWithPara
                echo "${ngvcAgentPrefix}: Ending Post Customization script, return value is: $?" >> $logPath
             else
                echo "${ngvcAgentPrefix}: Post Customization script owner is not root or the permission is not 700" >> $logPath
             fi
         else
             echo "${ngvcAgentPrefix}: Post Customization script does not exist.." >> $logPath
         fi
      else
         echo "${ngvcAgentPrefix}: Customization script folder($scriptfolder) is not under /etc/vmware.." >> $logPath
      fi
   else
      echo "${ngvcAgentPrefix}: PostCustomization does not config.." >> $logPath
   fi
}


#
# main
#
touch $logPath

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
define_constants
identify_distribution

echo "${ngvcAgentPrefix}: Running NGVC script..." >> $logPath

it=`vmware-rpctool "info-get guestinfo.it" 2>/dev/null`
if [ "$?" != "0" ]; then
   echo "${ngvcAgentPrefix}: Command get guestinfo.it failed or guestinfo.it do not exist..." >> $logPath
fi
if [ "$it" = "1" ]; then
   # set ngaFork = 1 when IT is ready
   vmware-rpctool "info-set guestinfo.ngaFork 1" >> $logPath 2>&1

   echo "${ngvcAgentPrefix}: It's in Internal Template..." >> $logPath

   # set customization state to sucess in IT stage
   vmware-rpctool "info-set guestinfo.it.CustomizationState success" >> $logPath 2>&1
   if [ "$?" != "0" ]; then
      echo "${ngvcAgentPrefix}: Setting guestinfo.it.CustomizationState failed..." >> $logPath
   fi
   sleep 1
   shutdown -h now
   exit 1
fi

replica=`vmware-rpctool "info-get guestinfo.replica" 2>/dev/null`
if [ "$?" != "0" ]; then
   echo "${ngvcAgentPrefix}: Command get guestinfo.replica failed or guestinfo.replica do not exist..." >> $logPath
fi

ngaFork=`vmware-rpctool "info-get guestinfo.ngaFork" 2>/dev/null`
if [ "$?" != "0" ]; then
   echo "${ngvcAgentPrefix}: Command get guestinfo.ngaFork failed or guestinfo.ngaFork do not exist..." >> $logPath
fi

forked=`vmware-rpctool "info-get guestinfo.forked" 2>/dev/null`
if [ "$?" != "0" ]; then
   echo "${ngvcAgentPrefix}: Command get guestinfo.forked failed or guestinfo.forked do not exist..." >> $logPath
fi

if [ "$ngaFork" = "1" -a "$forked" != "1" ]; then

   if [ "$replica" = '1' ]; then #{
      echo "${ngvcAgentPrefix}: This's in Replica..." >> $logPath
      vmware-rpctool "info-set guestinfo.replica.CustomizationState success" >> $logPath 2>&1
      sleep 3
      shutdown -h  now
      exit 2
   #}
   else #{ replica = '0'

      echo "${ngvcAgentPrefix}: This's Parent VM..." >> $logPath
      # set forked = 1, so this shell will not exec after reboot
      vmware-rpctool "info-set guestinfo.forked 1"

      # set customization state to success in parent stage
      vmware-rpctool "info-set guestinfo.parent.CustomizationState success"
      sleep 10

      icVer=`vmware-rpctool "info-get guestinfo.ic.version"`
      icCommand="vmfork-begin -1 -1"
      if [ "$icVer" = "GEN3" ]; then
         icCommand="instantclone.freeze"
         echo "${ngvcAgentPrefix}: This is GEN3" >> $logPath
      fi
      vmware-rpctool "${icCommand}"

      echo "${ngvcAgentPrefix}: This's Clone VM..." >> $logPath

      # set vmForked value into config file
      sed -i '/^'VMFORKED'/d' /etc/vmware/viewagent-config.txt  >> $logPath 2>&1
      echo "VMFORKED=1" >> /etc/vmware/viewagent-config.txt

      changeHWAddress
      changeHostName

      offlineJoinDomain=`awk '/^OfflineJoinDomain/{print}' /etc/vmware/viewagent-custom.conf | cut -f2 -d"="`
      if [ -z "$offlineJoinDomain" ]; then
         offlineJoinDomain="pbis"
      fi
      if [ "$offlineJoinDomain" = "pbis" ]; then
         echo "${ngvcAgentPrefix}: Beginning join domain offline..." >> $logPath
         offlineDomainJoin
         if [ "$?" != "0" ]; then
            echo "${ngvcAgentPrefix}: Failed to join domain offline..." >> $logPath
            customizationState="fail"
         fi
      else
         echo "${ngvcAgentPrefix}: Ignore PBIS offline join domain" >> $logPath
      fi

      runPostCustomizationScript

      # cleanup secret info
      vmware-rpctool "info-set guestinfo.machinePasswd none"
      vmware-rpctool "info-set guestinfo.machinePasswdChecksum none"

      # set customization state to success in clone stage
      vmware-rpctool "info-set guestinfo.clone.CustomizationState $customizationState"
      sed -i '/^'CUSTOMIZATION_STATE'/d' /etc/vmware/viewagent-config.txt  >> $logPath 2>&1
      if [ "$customizationState" = "success" ]; then
         echo "CUSTOMIZATION_STATE=true" >> /etc/vmware/viewagent-config.txt
         echo "${ngvcAgentPrefix}: Customization Done..." >> $logPath
      else
         echo "CUSTOMIZATION_STATE=false" >> /etc/vmware/viewagent-config.txt
         echo "${ngvcAgentPrefix}: Customization Failed..." >> $logPath
      fi

      sleep 3
      reboot

   fi #} $replica = '0'
else
   if [ "$forked" = "1" ]; then
      echo "${ngvcAgentPrefix}: This VM has been forked..." >> $logPath
   else
      echo "${ngvcAgentPrefix}: This VM is Master or non-cloned VM..." >> $logPath
   fi
fi
