#!/bin/bash

###################################################
# Copyright 2015 VMware, Inc.  All rights reserved.
###################################################

if [ "`id -u`" != "0" ]; then
   #
   # Requires root permission to run MonitorLoginUser.sh.
   #
   echo "MonitorLoginUser.sh must be executed as superuser"
   exit 1
fi

#
#Main
#

#
# Include common constants
#
. "`dirname $0`/commonlib.sh"
identify_distribution
define_constants

UserName=""
monitorID=""
errorNo=0

echo "vmMonitorLoginUser: start monitor the login user"
#
# Collect the login info.
# If the user logged in a domain, send the domainname\username to the standalone;
# else, send the hostname\username to the standalone.
#
hostName=$(hostname)
hostFQDN=$(hostname -f)

if [ "$hostName" = "$hostFQDN" ]; then
    # The user didn't log in a domain.
    hostInfo="$hostName"
else
    # The user logged in a domain.
    hostInfo=$(hostname -d)
    #
    # In some cases, the FQDN will be not same as the host name even the host is not joined a domain.
    # To handle above cases, we need to check the domain name is empty or not.
    # If yes, it means the user log in the local host.
    #
    if [ -z "$hostInfo" ]; then
       hostInfo="$hostName"
    fi
fi

echo "vmMonitorLoginUser: host name: $hostName, host info: $hostInfo"

while :
do
   UserName=""
   desktopType=""
   gnomePid=`pidof -s gnome-session`
   if [ "$?" = "0" ]; then
      UserName=`ps p $gnomePid -o uid --no-headers|awk '{print $1}'`
      desktopType="$GNOME_DESKTOP"
   fi

   matePid=`pidof -s mate-session`
   if [ "$?" = "0" ]; then
      UserName=`ps p $matePid -o uid --no-headers|awk '{print $1}'`
      desktopType="$MATE_DESKTOP"
   fi

   # Ubuntu 1604 should check gnome-session-binary proccess id to identify user login status.
   gnomeBinaryPid=`pidof -s gnome-session-binary`
   if [ "$?" = "0" ]; then
      UserName=`ps p $gnomeBinaryPid -o uid --no-headers|awk '{print $1}'`
      desktopType="$GNOME_DESKTOP"
   fi

   kdePid=`pidof -s ksmserver`
   if [ "$?" = "0" ]; then
      UserName=`ps p $kdePid -o uid --no-headers|awk '{print $1}'`
      # For KDE, It seems hard to forbid the suspend buton in UI. so do it in os level.
      # which means all user including root can't suspend. This is ok.
      chmod  -w /sys/power/state
      desktopType="$KDE_DESKTOP"
   fi

   xfce4Pid=`pidof xfce4-session`
   if [ "$?" = "0" ]; then
      UserName=`ps p $xfce4Pid -o uid --no-headers|awk '{print $1}'`
   fi

   # Check if there is user running gnome-session/mate-session
   if [ -z $UserName ]; then
       sleep 0.5
       continue
   fi

   # If the founded user name is UID, convert to user name
   user=`getent passwd $UserName|cut -d: -f1`
   if [ -z "$user" ]; then
       if [ "$errorNo" != "1" ]; then
          echo "vmMonitorLoginUser: Failed to convert $UserName to user name."
          errorNo=1
       fi
       sleep 0.5
       continue
   fi
   UserName=$user

   #
   # Check if the gnome-session/mate-session belong to gdm user
   # gnome-session is detected before user login for RHEL and CentOS
   # Here we try to handle such situation
   #
   if [ $UserName = "gdm" ]; then
       sleep 0.5
       continue
   fi

   # Check if envInfo exist for current user
   TMP_ENV_FILE="/var/vmware/view.$UserName/envInfo"
   if [ ! -f "$TMP_ENV_FILE" ]; then
       if [ "$errorNo" != "2" ]; then
          echo "vmMonitorLoginUser: The $TMP_ENV_FILE doesn't exist."
          errorNo=2
       fi
       sleep 0.5
       continue
   fi

   # Check VMW_MONITOR_ID
   newMonitorID=$(awk 'BEGIN{FS="="} ($1=="VMW_MONITOR_ID") {print $2}' "$TMP_ENV_FILE")
   if [ -z $newMonitorID ]; then
       if [ "$errorNo" != "3" ]; then
          echo "vmMonitorLoginUser: Can't get monitor ID from $TMP_ENV_FILE"
          errorNo=3
       fi
       sleep 0.5
       continue
   else
       if [ "$newMonitorID" = "$monitorID" ]; then
           sleep 0.5
           continue
       fi
   fi

   echo "vmMonitorLoginUser: $UserName login successful."

   # Record the new monitor ID
   monitorID=$newMonitorID

   # Record the desktop type
   TMP_DESKTOP_TYPE_FILE="/var/vmware/view.$UserName/desktopType"
   echo "$desktopType" > $TMP_DESKTOP_TYPE_FILE

   # Write hostinfo and username to logEvent
   TMP_LOGEVENT_FILE="/var/vmware/viewagent/logEvent"
   echo "$hostInfo $UserName" > $TMP_LOGEVENT_FILE

   # Write stop to greeter status file
   case "$DISTRO_ID" in
   ${DISTRO_ID_CENTOS}|\
   ${DISTRO_ID_RHEL_CLIENT}|\
   ${DISTRO_ID_RHEL_SERVER}|\
   ${DISTRO_ID_RHEL_WORKSTATION})
      if [ ${OS_MAJOR} -eq "6" ] || [ ${OS_MAJOR} -eq "7" ]; then
         check_greeter_status_file gdm
         echo "stop" > $VIEWAGENT_GREETER_STATUS_PATH/greeter_status
      fi
      ;;
   ${DISTRO_ID_SUSE})
      if [ ${OS_MAJOR} -eq "11" ]; then
         check_greeter_status_file gdm
         echo "stop" > $VIEWAGENT_GREETER_STATUS_PATH/greeter_status
      fi
      ;;
   ${DISTRO_ID_UBUNTU})
      if [ ${OS_MAJOR} -eq "14" ] || [ ${OS_MAJOR} -eq "16" ]; then
         check_greeter_status_file lightdm
         echo "stop" > $VIEWAGENT_GREETER_STATUS_PATH/greeter_status
      fi
      ;;
   *)
      ;;
   esac

   #
   # We need do a hard link of pulseaudio file for VMwareBlastServer.
   # But if $XDG_RUNTIME_DIR is in a separate partition, we can't do
   # this hard link out of $XDG_RUNTIME_DIR. We have to add read permit
   # to vmwblast account, so that VMwareBlastServer can access
   # $XDG_RUNTIME_DIR directly.
   #
   user_runtime_path=`grep -s --max-count=1 ^XDG_RUNTIME_DIR= "$TMP_ENV_FILE" | sed "s/XDG_RUNTIME_DIR=//"`
   if [ -n "$user_runtime_path" ] && [ -d "$user_runtime_path" ]; then
      partition_path=`df -P $user_runtime_path | awk 'END{print $NF}'`
      if [ "$partition_path" = "$user_runtime_path" ] && [ "`stat -c %U $user_runtime_path`" = "$UserName" ]; then
         chgrp vmwblast $user_runtime_path
         chmod g+rx $user_runtime_path
      fi
   fi
done
